@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-from _stories.returned import Success
 
 
 class BeginningOfStory(object):
@@ -9,11 +8,8 @@ class BeginningOfStory(object):
         self.parent_name = None
         self.same_object = None
 
-    def __call__(self, ctx):
-        return Success()
-
     @property
-    def __name__(self):
+    def story_name(self):
         if self.parent_name is None:
             return self.cls_name + "." + self.name
         elif self.same_object:
@@ -27,7 +23,4 @@ class BeginningOfStory(object):
 
 
 class EndOfStory(object):
-    def __call__(self, ctx):
-        return Success()
-
-    __name__ = "end_of_story"
+    pass
